package com.sybotan.service.utils

import com.alibaba.fastjson.JSON
import com.alibaba.fastjson.PropertyNamingStrategy
import com.alibaba.fastjson.serializer.SerializeConfig
import com.alibaba.fastjson.serializer.SerializerFeature
import java.io.InputStream
import java.lang.reflect.Type

object SJsonUtil {
    /** 序列化配置 */
    val serializeConfig = SerializeConfig()

    /**
     * 转换对象到json格式字符串
     *
     * @param   obj                     被转换对象
     * @param   namingStrategy          命名规则
     * @return  json格式字符串
     */
    fun toJson(obj: Any, namingStrategy: PropertyNamingStrategy? = null): String {
        return if (null == namingStrategy) {
            JSON.toJSONString(obj, serializeConfig, SerializerFeature.DisableCircularReferenceDetect)
        } else {
            val config = SerializeConfig()
            config.propertyNamingStrategy = namingStrategy
            JSON.toJSONString(obj, config, SerializerFeature.DisableCircularReferenceDetect)
        }
    } // Fun toJson()

    /**
     * 转换对象到json格式字符串，包含空字符串
     *
     * @param   obj                     被转换对象
     * @param   namingStrategy          首字母是否大写
     * @return  json格式字符串
     */
    fun toJsonAll(obj: Any, namingStrategy: PropertyNamingStrategy? = null): String {
        return if (null == namingStrategy) {
            JSON.toJSONString(obj,
                    serializeConfig,
                    SerializerFeature.WriteMapNullValue,
                    SerializerFeature.WriteNullListAsEmpty,
                    SerializerFeature.DisableCircularReferenceDetect)
        } else {
            val config = SerializeConfig()
            config.propertyNamingStrategy = namingStrategy
            JSON.toJSONString(obj,
                    config,
                    SerializerFeature.WriteMapNullValue,
                    SerializerFeature.WriteNullListAsEmpty,
                    SerializerFeature.DisableCircularReferenceDetect)
        }
    } // Fun toJsonAll()

    /**
     * 将Json数据解析成相应的映射对象
     *
     * @param   jsonData                要解析的JSON数据
     * @param   type                    Java类型
     * @return  解析结果
     */
    @Throws(Exception::class)
    fun <T> fromJson(jsonData: String, type: Class<T>): T? {
        return JSON.parseObject(jsonData, type)
    } // Fun fromJson()

    /**
     * 将Json数据解析成相应的映射对象
     *
     * @param   jsonData                要解析的JSON数据
     * @param   type                    Java类型
     * @return  解析结果
     */
    @Throws(Exception::class)
    fun fromJson(jsonData: String, type: Type): Any? {
        return JSON.parseObject(jsonData, type)
    } // Fun fromJson()

    /**
     * 将Json数据解析成相应的映射对象
     *
     * @param   input                   要解析的JSON数据输入流
     * @param   type                    Java类型
     * @param   namingStrategy          命名规则
     * @return  解析结果
     */
    @Throws(Exception::class)
    fun <T> fromJson(input: InputStream, type: Class<T>): T? {
        val count = input.available()
        val buf = ByteArray(count)
        var readCount = 0
        while (readCount < count) {
            readCount += input.read(buf, readCount, count - readCount)
        }

        return fromJson(String(buf), type)
    } // Fun fromJson()

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /**
     * 将Json数据解析成相应的映射对象
     *
     * @param   jsonData                要解析的JSON数据
     * @param   namingStrategy          命名规则
     * @return  解析结果
     */
    @Throws(Exception::class)
    inline fun <reified T> fromJson(jsonData: String, namingStrategy: PropertyNamingStrategy? = null): T {
        return JSON.parseObject(jsonData, T::class.java)
    } // Fun parseJson()

    /**
     * 将Json数据解析成相应的映射对象
     *
     * @param   input                   要解析的JSON数据输入流
     * @param   namingStrategy          命名规则
     * @return  解析结果
     */
    @Throws(Exception::class)
    inline fun <reified T> fromJson(input: InputStream, namingStrategy: PropertyNamingStrategy? = null): T {
        val count = input.available()
        val buf = ByteArray(count)
        var readCount = 0
        while (readCount < count) {
            readCount += input.read(buf, readCount, count - readCount)
        }

        return fromJson(String(buf), namingStrategy)
    } // Fun fromJson()
} // Object SJsonUtil