/*
 * ********************************************************************************************************************
 *
 *               iFHS7.
 *              ;BBMBMBMc                  rZMBMBR              BMB
 *              MBEr:;PBM,               7MBMMEOBB:             BBB                       RBW
 *     XK:      BO     SB.     :SZ       MBM.       c;;     ir  BBM :FFr       :SSF:    ;xBMB:r   iuGXv.    i:. iF2;
 *     DBBM0r.  :D     S7   ;XMBMB       GMBMu.     MBM:   BMB  MBMBBBMBMS   WMBMBMBBK  MBMBMBM  BMBRBMBW  .MBMBMBMBB
 *      :JMRMMD  ..    ,  1MMRM1;         ;MBMBBR:   MBM  ;MB:  BMB:   MBM. RMBr   sBMH   BM0         UMB,  BMB.  KMBv
 *     ;.   XOW  B1; :uM: 1RE,   i           .2BMBs  rMB. MBO   MBO    JMB; MBB     MBM   BBS    7MBMBOBM:  MBW   :BMc
 *     OBRJ.SEE  MRDOWOR, 3DE:7OBM       .     ;BMB   RMR7BM    BMB    MBB. BMB    ,BMR  .BBZ   MMB   rMB,  BMM   rMB7
 *     :FBRO0D0  RKXSXPR. JOKOOMPi       BMBSSWBMB;    BMBB:    MBMB0ZMBMS  .BMBOXRBMB    MBMDE RBM2;SMBM;  MBB   xBM2
 *         iZGE  O0SHSPO. uGZ7.          sBMBMBDL      :BMO     OZu:BMBK,     rRBMB0;     ,EBMB  xBMBr:ER.  RDU   :OO;
 *     ,BZ, 1D0  RPSFHXR. xWZ .SMr                  . .BBB
 *      :0BMRDG  RESSSKR. 2WOMBW;                   BMBMR
 *         i0BM: SWKHKGO  MBDv
 *           .UB  OOGDM. MK,                                          Copyright (c) 2015-2019.  斯伯坦机器人
 *              ,  XMW  ..
 *                  r                                                                     All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.sybotan.service.utils.crypt

import javax.crypto.BadPaddingException
import javax.crypto.Cipher
import javax.crypto.IllegalBlockSizeException
import javax.crypto.NoSuchPaddingException
import java.io.IOException
import java.security.*
import java.security.interfaces.RSAPrivateKey
import java.security.interfaces.RSAPublicKey
import java.security.spec.InvalidKeySpecException
import java.security.spec.PKCS8EncodedKeySpec
import java.security.spec.X509EncodedKeySpec
import java.util.*

/**
 * RSA安全编码组件
 *
 * @author Andy
 */
abstract class SRSACoderUtil : SCoderUtil() {
    companion object {
        const val KEY_ALGORITHM = "RSA"
        const val SIGNATURE_ALGORITHM = "MD5withRSA"

        private const val PUBLIC_KEY = "RSAPublicKey"
        private const val PRIVATE_KEY = "RSAPrivateKey"

        /**
         * 用私钥对信息生成数字签名
         *
         * @param data       加密数据
         * @param privateKey 私钥
         * @return
         * @throws IOException
         * @throws NoSuchAlgorithmException
         * @throws InvalidKeySpecException
         * @throws InvalidKeyException
         * @throws SignatureException
         */
        @Throws(IOException::class, NoSuchAlgorithmException::class, InvalidKeySpecException::class, InvalidKeyException::class, SignatureException::class)
        fun sign(data: ByteArray, privateKey: String): String? {
            // 解密由base64编码的私钥
            val keyBytes = Base64.getDecoder().decode(privateKey)

            // 构造PKCS8EncodedKeySpec对象
            val pkcs8KeySpec = PKCS8EncodedKeySpec(keyBytes!!)

            // KEY_ALGORITHM 指定的加密算法
            val keyFactory = KeyFactory.getInstance(KEY_ALGORITHM)

            // 取私钥匙对象
            val priKey = keyFactory.generatePrivate(pkcs8KeySpec)

            // 用私钥对信息生成数字签名
            val signature = Signature.getInstance(SIGNATURE_ALGORITHM)
            signature.initSign(priKey)
            signature.update(data)

            return Base64.getEncoder().encodeToString(signature.sign())
        } // Fun sign()

        /**
         * 校验数字签名
         *
         * @param data      加密数据
         * @param publicKey 公钥
         * @param sign      数字签名
         * @return 校验成功返回true 失败返回false
         * @throws IOException
         * @throws NoSuchAlgorithmException
         * @throws InvalidKeySpecException
         * @throws InvalidKeyException
         * @throws SignatureException
         */
        @Throws(IOException::class, NoSuchAlgorithmException::class, InvalidKeySpecException::class, InvalidKeyException::class, SignatureException::class)
        fun verify(data: ByteArray, publicKey: String, sign: String): Boolean {

            // 解密由base64编码的公钥
            val keyBytes = Base64.getDecoder().decode(publicKey)

            // 构造X509EncodedKeySpec对象
            val keySpec = X509EncodedKeySpec(keyBytes!!)

            // KEY_ALGORITHM 指定的加密算法
            val keyFactory = KeyFactory.getInstance(KEY_ALGORITHM)

            // 取公钥匙对象
            val pubKey = keyFactory.generatePublic(keySpec)

            val signature = Signature.getInstance(SIGNATURE_ALGORITHM)
            signature.initVerify(pubKey)
            signature.update(data)

            // 验证签名是否正常
            return signature.verify(Base64.getDecoder().decode(sign))
        } // Fun verify()

        /**
         * 解密<br></br>
         * 用私钥解密
         *
         * @param data
         * @param key
         * @return
         * @throws IOException
         * @throws NoSuchAlgorithmException
         * @throws InvalidKeySpecException
         * @throws NoSuchPaddingException
         * @throws InvalidKeyException
         * @throws BadPaddingException
         * @throws IllegalBlockSizeException
         */
        @Throws(IOException::class, NoSuchAlgorithmException::class, InvalidKeySpecException::class, NoSuchPaddingException::class, InvalidKeyException::class, IllegalBlockSizeException::class, BadPaddingException::class)
        fun decryptByPrivateKey(data: ByteArray, key: String): ByteArray {
            // 对密钥解密
            val keyBytes = Base64.getDecoder().decode(key)

            // 取得私钥
            val pkcs8KeySpec = PKCS8EncodedKeySpec(keyBytes!!)
            val keyFactory = KeyFactory.getInstance(KEY_ALGORITHM)
            val privateKey = keyFactory.generatePrivate(pkcs8KeySpec)

            // 对数据解密
            val cipher = Cipher.getInstance(keyFactory.algorithm)
            cipher.init(Cipher.DECRYPT_MODE, privateKey)

            return cipher.doFinal(data)
        } // Fun decryptByPrivateKey()

        /**
         * 解密<br></br>
         * 用公钥解密
         *
         * @param data
         * @param key
         * @return
         * @throws IOException
         * @throws NoSuchAlgorithmException
         * @throws InvalidKeySpecException
         * @throws NoSuchPaddingException
         * @throws InvalidKeyException
         * @throws BadPaddingException
         * @throws IllegalBlockSizeException
         */
        @Throws(IOException::class, NoSuchAlgorithmException::class, InvalidKeySpecException::class, NoSuchPaddingException::class, InvalidKeyException::class, IllegalBlockSizeException::class, BadPaddingException::class)
        fun decryptByPublicKey(data: ByteArray, key: String): ByteArray {
            // 对密钥解密
            val keyBytes = Base64.getDecoder().decode(key)

            // 取得公钥
            val x509KeySpec = X509EncodedKeySpec(keyBytes!!)
            val keyFactory = KeyFactory.getInstance(KEY_ALGORITHM)
            val publicKey = keyFactory.generatePublic(x509KeySpec)

            // 对数据解密
            val cipher = Cipher.getInstance(keyFactory.algorithm)
            cipher.init(Cipher.DECRYPT_MODE, publicKey)

            return cipher.doFinal(data)
        } // Fun decryptByPublicKey()

        /**
         * 用公钥加密
         *
         * @param data
         * @param key
         * @return
         * @throws IOException
         * @throws NoSuchAlgorithmException
         * @throws InvalidKeySpecException
         * @throws NoSuchPaddingException
         * @throws InvalidKeyException
         * @throws BadPaddingException
         * @throws IllegalBlockSizeException
         */
        @Throws(IOException::class, NoSuchAlgorithmException::class, InvalidKeySpecException::class, NoSuchPaddingException::class, InvalidKeyException::class, IllegalBlockSizeException::class, BadPaddingException::class)
        fun encryptByPublicKey(data: ByteArray, key: String): ByteArray {
            // 对公钥解密
            val keyBytes = Base64.getDecoder().decode(key)

            // 取得公钥
            val x509KeySpec = X509EncodedKeySpec(keyBytes!!)
            val keyFactory = KeyFactory.getInstance(KEY_ALGORITHM)
            val publicKey = keyFactory.generatePublic(x509KeySpec)

            // 对数据加密
            val cipher = Cipher.getInstance(keyFactory.algorithm)
            cipher.init(Cipher.ENCRYPT_MODE, publicKey)

            return cipher.doFinal(data)
        } // Fun encryptByPublicKey()

        /**
         * 用私钥加密
         *
         * @param data
         * @param key
         * @return
         * @throws IOException
         * @throws NoSuchAlgorithmException
         * @throws InvalidKeySpecException
         * @throws NoSuchPaddingException
         * @throws InvalidKeyException
         * @throws BadPaddingException
         * @throws IllegalBlockSizeException
         */
        @Throws(IOException::class, NoSuchAlgorithmException::class, InvalidKeySpecException::class, NoSuchPaddingException::class, InvalidKeyException::class, IllegalBlockSizeException::class, BadPaddingException::class)
        fun encryptByPrivateKey(data: ByteArray, key: String): ByteArray {
            // 对密钥解密
            val keyBytes = Base64.getDecoder().decode(key)

            // 取得私钥
            val pkcs8KeySpec = PKCS8EncodedKeySpec(keyBytes!!)
            val keyFactory = KeyFactory.getInstance(KEY_ALGORITHM)
            val privateKey = keyFactory.generatePrivate(pkcs8KeySpec)

            // 对数据加密
            val cipher = Cipher.getInstance(keyFactory.algorithm)
            cipher.init(Cipher.ENCRYPT_MODE, privateKey)

            return cipher.doFinal(data)
        } // Fun encryptByPrivateKey()

        /**
         * 取得私钥
         *
         * @param keyMap
         * @return
         */
        fun getPrivateKey(keyMap: Map<String, Any>): String? {
            val key = keyMap[PRIVATE_KEY] as Key

            return Base64.getEncoder().encodeToString(key.encoded)
        } // Fun getPrivateKey()

        /**
         * 取得公钥
         *
         * @param keyMap
         * @return
         */
        fun getPublicKey(keyMap: Map<String, Any>): String? {
            val key = keyMap[PUBLIC_KEY] as Key

            return Base64.getEncoder().encodeToString(key.encoded)
        } // Fun getPublicKey()

        /**
         * 初始化密钥
         *
         * @return
         * @throws NoSuchAlgorithmException
         */
        @Throws(NoSuchAlgorithmException::class)
        fun initKey(): Map<String, Any> {
            val keyPairGen = KeyPairGenerator.getInstance(KEY_ALGORITHM)
            keyPairGen.initialize(1024)

            val keyPair = keyPairGen.generateKeyPair()

            // 公钥
            val publicKey = keyPair.public as RSAPublicKey

            // 私钥
            val privateKey = keyPair.private as RSAPrivateKey

            val keyMap = HashMap<String, Any>(2)

            keyMap[PUBLIC_KEY] = publicKey
            keyMap[PRIVATE_KEY] = privateKey
            return keyMap
        } // Fun initKey()
    }
} // Class SRSACoderUtil
