/*
 * ********************************************************************************************************************
 *
 *               iFHS7.
 *              ;BBMBMBMc                  rZMBMBR              BMB
 *              MBEr:;PBM,               7MBMMEOBB:             BBB                       RBW
 *     XK:      BO     SB.     :SZ       MBM.       c;;     ir  BBM :FFr       :SSF:    ;xBMB:r   iuGXv.    i:. iF2;
 *     DBBM0r.  :D     S7   ;XMBMB       GMBMu.     MBM:   BMB  MBMBBBMBMS   WMBMBMBBK  MBMBMBM  BMBRBMBW  .MBMBMBMBB
 *      :JMRMMD  ..    ,  1MMRM1;         ;MBMBBR:   MBM  ;MB:  BMB:   MBM. RMBr   sBMH   BM0         UMB,  BMB.  KMBv
 *     ;.   XOW  B1; :uM: 1RE,   i           .2BMBs  rMB. MBO   MBO    JMB; MBB     MBM   BBS    7MBMBOBM:  MBW   :BMc
 *     OBRJ.SEE  MRDOWOR, 3DE:7OBM       .     ;BMB   RMR7BM    BMB    MBB. BMB    ,BMR  .BBZ   MMB   rMB,  BMM   rMB7
 *     :FBRO0D0  RKXSXPR. JOKOOMPi       BMBSSWBMB;    BMBB:    MBMB0ZMBMS  .BMBOXRBMB    MBMDE RBM2;SMBM;  MBB   xBM2
 *         iZGE  O0SHSPO. uGZ7.          sBMBMBDL      :BMO     OZu:BMBK,     rRBMB0;     ,EBMB  xBMBr:ER.  RDU   :OO;
 *     ,BZ, 1D0  RPSFHXR. xWZ .SMr                  . .BBB
 *      :0BMRDG  RESSSKR. 2WOMBW;                   BMBMR
 *         i0BM: SWKHKGO  MBDv
 *           .UB  OOGDM. MK,                                          Copyright (c) 2015-2019.  斯伯坦机器人
 *              ,  XMW  ..
 *                  r                                                                     All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.sybotan.service.kafka

import com.sybotan.base.extensions.toJson
import org.springframework.kafka.core.KafkaTemplate
import org.springframework.kafka.support.SendResult
import org.springframework.util.concurrent.ListenableFuture
import org.springframework.util.concurrent.ListenableFutureCallback
import java.util.*

/**
 * Kafka 服务。用于连接 kafka, 收发 kafka 消息
 *
 * @author  庞利祥 <sybotan@126.com>
 */
open class SKafkaService {
    /** 延迟发送时间 */
    var delaySendTime = 3000L
    /** Kafka 访问模板 */
    @Suppress("UNCHECKED_CAST")
    lateinit var kafkaTemplate: KafkaTemplate<Int, String>
    /** 实现迟延相关数据 */
    private val delayMap = HashMap<String, SDelayData>()

    /**
     * 发送通知
     *
     * @param   topic       发送标题
     * @param   notice      通知
     * @param   delaySend   是否启动延迟发送，默认延迟发送
     */
    fun sendNotice(topic: String, notice: SBaseNotice, delaySend: Boolean = true) {
        val delayData = delayMap[topic] ?: SDelayData()
        if (!delayMap.containsKey(topic)) {
            delayMap[topic] = delayData
        }

        // 合并请求
        if (!mergeNotice(topic, notice)) {
            if (delayData.lastNotice != null) {
                sendTopic(topic, delayData.lastNotice!!.toJson())
            }
            delayData.lastNotice = notice
        }

        // 是否延迟发送
        if (delaySend) {
            delaySend(topic)
        } else {
            flush(topic)
        }

        return
    } // Fun sendNotice()

    /**
     * 发送 topic 消息
     *
     * @param   topic       发送标题
     * @param   data        消息数据
     */
    fun sendTopic(topic: String, data: String) {
        val future: ListenableFuture<SendResult<Int, String>> = kafkaTemplate.send(topic, data).also {
            it.addCallback(object : ListenableFutureCallback<SendResult<Int, String>> {
                /**
                 * 发送失败回调
                 */
                override fun onFailure(ex: Throwable) {
                } // Fun onFailure
                /**
                 * 发送成功回调
                 */
                override fun onSuccess(result: SendResult<Int, String>) {
                } // Fun onSuccess()
            })
        }

        kafkaTemplate.flush()
    } // Fun sendTopic()

    /**
     * 合并通知
     *
     * @param   lastNotice          缓存的通知
     * @param   notice              新的通知
     * @return  成功返回true，否则返回false
     */
    open fun mergeNotice(lastNotice: SBaseNotice, notice: SBaseNotice): Boolean {
        return false
    } // Function mergeNotice()

    /**
     * 延迟发送缓存的消息
     *
     * @param   topic       发送标题
     */
    private fun delaySend(topic: String) {
        val delayData = delayMap[topic] ?: return

        // 如果延迟任务不为空，则取消之前的延迟任务。保证延迟任务在最后一次发送后执行。
        if (delayData.delayTask != null) {
            delayData.delayTask!!.cancel()
        }

        // 创建延迟任务
        delayData.delayTask = object : TimerTask() {
            override fun run() {
                // 发送缓存的任务
                flush(topic)
            }
        }

        // 启动延迟发送任务
        delayData.delayTimer.schedule(delayData.delayTask, delaySendTime)

        return
    } // Fun delaySend()

    /**
     * 发送缓存的消息
     *
     * @param   topic       发送标题
     */
    fun flush(topic: String) {
        val delayData = delayMap[topic] ?: return
        // 如果没有缓存通知
        if (delayData.lastNotice == null) {
            return
        }

        sendTopic(topic, delayData.lastNotice!!.toJson())
        delayData.lastNotice = null

        return
    } // Fun flush()

    /**
     * 合并通知
     *
     * @param   topic       发送标题
     * @param   notice      要合并的通知
     * @return  成功返回true，否则返回false
     */
    private fun mergeNotice(topic: String, notice: SBaseNotice): Boolean {
        val delayData = delayMap[topic] ?: return false

        // 如果最后发送的通知为空
        if (delayData.lastNotice == null) {
            delayData.lastNotice = notice
            return true
        }

        return mergeNotice(delayData.lastNotice!!, notice)
    } // Fun mergeNotice()

    /**
     * 延迟处理数据
     */
    inner class SDelayData {
        /** 最后发送的通知 */
        var lastNotice: SBaseNotice? = null
        /** 延迟发送通知定时器 */
        val delayTimer = Timer()
        /** 延迟发送通知定时任务 */
        var delayTask: TimerTask? = null
    } // Class SDelayData
} // Class SKafkaService
